#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only

from time import sleep as slp
from youqu3 import logger
from youqu3 import setting


def sleep(second: [float, int]):
    """
    重写sleep方法
    1.增加等待时间的日志
    2.根据不同CPU架构进行放大
    :param second: 等待时间
    :return:
    """
    multiple = getattr(setting.Sleepx, setting.SYS_ARCH)
    mult_sec = second * multiple
    logger.debug(f"sleep {second} s [{setting.SYS_ARCH} * {multiple} = {mult_sec}]")
    slp(mult_sec)

if __name__ == '__main__':
    sleep(1)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import functools
import os.path
import pathlib
import shutil
import time

from youqu3 import logger
from youqu3 import setting
from youqu3.cmd import Cmd, RemoteCmd


def exclude():
    exclude_files = [
        "report",
        "__pycache__",
        ".pytest_cache",
        ".vscode",
        ".idea",
        ".git",
    ]

    exclude_str = ""
    for i in exclude_files:
        exclude_str += f"--exclude='{i}' "

    return exclude_str


def server_rootdir(project_abspath: str = None):
    path = "." if project_abspath is None else project_abspath
    return pathlib.Path(path).absolute()


def client_rootdir(user, project_abspath):
    return f"/home/{user}/_rpc_{server_rootdir(project_abspath).name}"


def transfer_to_client(ip, password, user, project_abspath):
    rsync = 'rsync -av -e "ssh -o StrictHostKeyChecking=no"'
    Cmd.run(f"rm -rf ~/.ssh/known_hosts")
    remote_cmd = RemoteCmd(user, ip, password)
    remote_cmd.remote_run(f"mkdir -p {client_rootdir(user, project_abspath)}")
    Cmd.expect_run(
        f"/bin/bash -c '{rsync} {exclude()} {server_rootdir(project_abspath)}/* {user}@{ip}:{client_rootdir(user, project_abspath)}/'",
        events={'password': f'{password}\n'},
        return_code=True
    )
    Cmd.expect_run(
        f"/bin/bash -c '{rsync} {exclude()} {server_rootdir(project_abspath)}/.env {user}@{ip}:{client_rootdir(user, project_abspath)}/'",
        events={'password': f'{password}\n'},
        return_code=True
    )
    from invoke.exceptions import UnexpectedExit
    try:
        stdout, return_code = remote_cmd.remote_run(f"cd {client_rootdir(user, project_abspath)}/ && ls env_ok",
                                                    return_code=True)
    except UnexpectedExit:
        _, return_code = remote_cmd.remote_run(
            "export PATH=$PATH:$HOME/.local/bin;"
            f"cd {client_rootdir(user, project_abspath)} && youqu3 envx && touch env_ok",
            return_code=True
        )
        logger.info(f"RPC环境安装{'成功' if return_code == 0 else '失败'} - < {user}@{ip} >")


def start_client_service(ip, password, user, filename, project_abspath):
    service_name = f"{filename}.service"
    remote_cmd = RemoteCmd(user, ip, password)
    from invoke.exceptions import UnexpectedExit
    try:
        remote_cmd.remote_run(f"ls /lib/systemd/system/{service_name}")
    except UnexpectedExit:
        remote_cmd.remote_sudo_run(
            f"cp {os.path.join(client_rootdir(user, project_abspath), service_name)} /lib/systemd/system/")
        remote_cmd.remote_sudo_run(f"chmod 644 /lib/systemd/system/{service_name}")
        remote_cmd.remote_sudo_run(f"systemctl daemon-reload")
        remote_cmd.remote_sudo_run(f"systemctl restart {service_name}")
        time.sleep(1)


def restart_client_service(ip, password, user, filename):
    RemoteCmd(user, ip, password).remote_sudo_run(f"sudo systemctl restart {filename}.service")
    time.sleep(1)


def gen_service_file(user, filename, project_abspath):
    service_file = pathlib.Path(project_abspath).absolute() / f"{filename}.service"
    service_py = setting.RPC_PATH / f"{filename}.py"
    if service_file.exists() and os.path.exists(os.path.join(project_abspath, f"{filename}.py")):
        return

    tpl_file = setting.RPC_PATH / "rpc_tpl.service"
    with open(tpl_file, "r", encoding="utf-8") as sf:
        service_tmp = sf.read()
    service_tmp.format(
        user=user,
        client_rootdir=client_rootdir(user, project_abspath),
        start_service=f"pipenv run python {filename}"
    )
    with open(service_file, "w", encoding="utf-8") as sf:
        sf.write(
            service_tmp.format(
                user=user,
                client_rootdir=client_rootdir(user, project_abspath),
                start_service=f"pipenv run python {filename}.py"
            )
        )
    shutil.copyfile(service_py, os.path.join(project_abspath, f"{filename}.py"))


def guard_rpc(filename):
    def deco(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            user = kwargs.get("user") or args[0]
            ip = kwargs.get("ip") or args[1]
            if not user or not ip:
                raise ValueError("user and ip are required")
            password = kwargs.get("password")
            project_abspath = kwargs.get("project_abspath")
            auto_restart = kwargs.get("auto_restart")
            return_code = None
            try:
                _, return_code = RemoteCmd(user, ip, password).remote_sudo_run(f"systemctl status {filename}.service")
            except BaseException:
                gen_service_file(user, filename, project_abspath)
                transfer_to_client(ip, password, user, project_abspath)
                start_client_service(ip, password, user, filename, project_abspath)
            if return_code != 0:
                gen_service_file(user, filename, project_abspath)
                transfer_to_client(ip, password, user, project_abspath)
                start_client_service(ip, password, user, filename, project_abspath)
            else:
                Cmd.run(f"rm -f {project_abspath}/{filename}.py")
                Cmd.run(f"rm -f {project_abspath}/{filename}.service")
            if auto_restart:
                restart_client_service(ip, password, user, filename)
            res = func(*args, **kwargs)
            return res

        return wrapper

    return deco



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
from funnylog2.config import config as funnylog2_config
from typing import Union
from youqu3 import exceptions
from youqu3 import log, logger, setting
from youqu3.cmd import Cmd

funnylog2_config.CLASS_NAME_ENDSWITH.append("Assert")


@log
class Assert:
    """
    自定义断言类
    """

    @staticmethod
    def assert_image_exist(
        widget: str,
        rate: float = None,
        multiple: bool = False,
        picture_abspath: str = None,
        network_retry: int = None,
        pause: [int, float] = None,
        timeout: [int, float] = None,
        match_number: int = None,
    ):
        """判断界面存在{{widget}}模板图片"""
        from youqu3.gui import pylinuxauto

        try:
            pylinuxauto.find_element_by_image(
                widget,
                rate=rate,
                multiple=multiple,
                picture_abspath=picture_abspath,
                network_retry=network_retry,
                pause=pause,
                timeout=timeout,
                max_match_number=match_number,
            )
        except pylinuxauto.exceptions.TemplateElementNotFound as exc:
            raise AssertionError(
                exc
            ) from pylinuxauto.exceptions.TemplateElementNotFound

    @staticmethod
    def assert_image_not_exist(
        widget: str,
        rate: float = None,
        multiple: bool = False,
        picture_abspath: str = None,
        network_retry: int = None,
        pause: [int, float] = None,
        timeout: [int, float] = None,
        match_number: int = None,
    ):
        """判断界面不存在{{widget}}模板图片"""
        from youqu3.gui import pylinuxauto

        try:
            pylinuxauto.find_element_by_image(
                widget,
                rate=rate,
                multiple=multiple,
                picture_abspath=picture_abspath,
                network_retry=network_retry,
                pause=pause,
                timeout=timeout,
                max_match_number=match_number,
            )
            raise exceptions.TemplateElementFound(widget)
        except pylinuxauto.exceptions.TemplateElementNotFound:
            ...

    @classmethod
    def assert_image_exist_during_time(
        cls,
        widget: str,
        screen_time: Union[float, int],
        rate: float = None,
        pause: Union[int, float] = None,
    ):
        """
        在一段时间内截图多张图片进行识别，其中有一张图片识别成功即返回结果;
        适用于气泡类的断言，比如气泡在1秒内消失，如果用常规的图像识别则有可能无法识别到；
        :param image_path: 要识别的模板图片；
        :param screen_time: 截取屏幕图片的时间，单位秒；
        :param rate: 识别率；
        :param pause: 截取屏幕图片的间隔时间，默认不间隔；
        """
        logger.info(
            f"屏幕上匹配图片< {f'***{widget[-40:]}' if len(widget) >= 40 else widget} >"
        )
        from youqu3.gui import pylinuxauto

        try:
            pylinuxauto.get_during(widget, screen_time, rate, pause)
        except exceptions.TemplateElementNotFound as exc:
            raise AssertionError(exc) from exceptions.TemplateElementNotFound

    @staticmethod
    def assert_file_exist(file_path):
        """判断文件{{file_path}}存在"""
        if not os.path.exists(os.path.expanduser(file_path)):
            raise AssertionError(f"文件{file_path}不存在")
        return True

    @staticmethod
    def assert_file_not_exist(file_path):
        """判断文件{{file_path}}不存在"""
        if os.path.exists(os.path.expanduser(file_path)):
            raise AssertionError(f"文件{file_path}存在")

    @staticmethod
    def assert_element_exist(expr):
        """判断元素{{expr}}存在"""
        from youqu3.gui import pylinuxauto

        try:
            pylinuxauto.find_element_by_attr_path(expr)
        except pylinuxauto.exceptions.ElementNotFound:
            raise exceptions.ElementNotFound(expr)

    @staticmethod
    def assert_element_not_exist(expr):
        """判断元素{{expr}}不存在"""
        from youqu3.gui import pylinuxauto

        try:
            pylinuxauto.find_element_by_attr_path(expr)
            raise exceptions.ElementFound(expr)
        except pylinuxauto.exceptions.ElementNotFound:
            pass

    @staticmethod
    def assert_equal(expect, actual):
        """判断预期值<{{expect}>与实际值<{{actual}>相等"""
        if expect != actual:
            raise AssertionError(f"预期值<{expect}>与实际值<{actual}>不相等")

    @staticmethod
    def assert_not_equal(expect, actual):
        """判断预期值<{{expect}>与实际值<{{actual}>不相等"""
        if expect == actual:
            raise AssertionError(f"预期值<{expect}>与实际值<{actual}>相等")

    @staticmethod
    def assert_in(target: str, pool: str):
        """判断<{{target}}>在<{{pool}}>中"""
        if target not in pool:
            raise AssertionError(f"<{target}>不在<{pool}>中")

    @staticmethod
    def assert_not_in(target: str, pool: str):
        """判断<{{target}}>不在<{{pool}}>中"""
        if target in pool:
            raise AssertionError(f"<{target}>在<{pool}>中")

    @staticmethod
    def assert_sequence_in(target: list, pool: list):
        """判断<{{target}}>在<{{pool}}>中"""
        for i in target:
            if i not in pool:
                raise AssertionError(f"{pool}中不存在{i}")

    @staticmethod
    def assert_sequence_not_in(target: list, pool: list):
        """判断<{{target}}>不在<{{pool}}>中"""
        for i in target:
            if i in pool:
                raise AssertionError(f"{pool}中存在{i}")

    @staticmethod
    def assert_true(expect):
        """断言{{expect}}结果为真"""
        if not expect:
            raise AssertionError(f"<{expect}>不为真")

    @staticmethod
    def assert_false(expect):
        """断言{{expect}}结果为假"""
        if expect:
            raise AssertionError(f"<{expect}>不为假")

    @staticmethod
    def assert_any(expect):
        """断言任一{{expect}}结果为真"""
        if not any(expect):
            raise AssertionError(f"<{expect}>均不为真")

    @staticmethod
    def assert_all(expect):
        """断言所有{{expect}}结果为真"""
        if not all(expect):
            raise AssertionError(f"<{expect}>不均为真")

    @staticmethod
    def assert_ocr_exist(
        *args,
        picture_abspath=None,
        similarity=0.6,
        return_first=False,
        lang="ch",
        network_retry: int = None,
        pause: [int, float] = None,
        timeout: [int, float] = None,
        max_match_number: int = None,
        mode: str = "all",
        bbox: dict = None,
    ):
        """
        断言文案存在
        :param args:
            目标字符,识别一个字符串或多个字符串,并返回其在图片中的坐标;
            如果不传参，返回图片中识别到的所有字符串。
        :param picture_abspath: 要识别的图片路径，如果不传默认截取全屏识别。
        :param similarity: 匹配度。
        :param return_first: 只返回第一个,默认为 False,返回识别到的所有数据。
        :param lang: `ch`, `en`, `fr`, `german`, `korean`, `japan`
        :param network_retry: 连接服务器重试次数
        :param pause: 重试间隔时间,单位秒
        :param timeout: 最大匹配超时,单位秒
        :param max_match_number: 最大匹配次数
        :param mode: "all" or "any"，all 表示识别所有目标字符，any 表示识别任意一个目标字符，默认值为 all
        :param bbox:
            接收一个字典，包含一个区域，在区域内进行识别，用于干扰较大时提升OCR识别精准度
            字典字段:
                start_x: 开始 x 坐标（左上角）
                start_y: 开始 y 坐标（左上角）
                w: 宽度
                h: 高度
                end_x: 结束 x 坐标（右下角）
                end_y: 结束 y 坐标（右下角）
                return_one: True/False（默认为True）
                注意 ：
                    1、end_x + end_y 与 w + h 为互斥关系, 必须且只能传入其中一组
                    2、加入此参数后代表范围内只能出现唯一坐标则通过，若必须返回多组坐标可修改'return_one'的值
                示例：
                {"start_x": 0, "start_y": 0, "w": 100, "h": 100}
                {"start_x": 0, "start_y": 0, "end_x": 100, "end_y": 100}
        """
        if len(args) == 0:
            raise ValueError("缺少 ocr 断言关键字")

        pic = None
        if picture_abspath is not None:
            pic = picture_abspath

        from youqu3.gui import pylinuxauto
        from youqu3.gui import pylinuxauto_config

        res = pylinuxauto.find_element_by_ocr(
            *args,
            picture_abspath=pic,
            similarity=similarity,
            return_first=return_first,
            lang=lang,
            network_retry=network_retry,
            pause=pause,
            timeout=timeout,
            max_match_number=max_match_number,
            bbox=bbox,
        )
        if res.result is False:
            raise AssertionError(
                (
                    f"通过OCR未识别到：{args}",
                    f"{pic if pic else pylinuxauto_config.SCREEN_CACHE}",
                )
            )
        else:
            if isinstance(res.result, tuple):
                pass
            elif isinstance(res.result, dict):
                mode = mode.lower()
                if mode == "all" and False in res.result.values():
                    res = filter(lambda x: x[1] is False, res.result.items())
                    raise AssertionError(
                        (
                            f"通过OCR未识别到：{dict(res)} <- {args}",
                            f"{pic if pic else pylinuxauto_config.SCREEN_CACHE}",
                        )
                    )
                elif mode == "any" and len(res.result) == list(
                    res.result.values()
                ).count(False):
                    raise AssertionError(
                        (
                            f"通过OCR未识别到：{args}中的任意一个",
                            f"{pic if pic else pylinuxauto_config.SCREEN_CACHE}",
                        )
                    )
                else:
                    if bbox is not None:
                        if bbox.get("return_one") is True:
                            new_res = {}
                            for key, value in res.items():
                                new_res[key] = (
                                    value[0] + bbox.get("start_x"),
                                    value[1] + bbox.get("start_y"),
                                )
                            raise AssertionError(
                                f"传入bbox参数，范围内关键词非唯一：{new_res}"
                            )
                        elif bbox.get("return_one") is False:
                            pass

    @staticmethod
    def assert_ocr_not_exist(
        *args,
        picture_abspath=None,
        similarity=0.6,
        return_first=False,
        lang="ch",
        network_retry: int = None,
        pause: [int, float] = None,
        timeout: [int, float] = None,
        max_match_number: int = None,
        bbox: dict = None,
    ):
        """断言文案不存在"""
        if len(args) == 0:
            raise ValueError("缺少 ocr 断言关键字")

        pic = None
        if picture_abspath is not None:
            pic = picture_abspath

        from youqu3.gui import pylinuxauto
        from youqu3.gui import pylinuxauto_config

        res = pylinuxauto.find_element_by_ocr(
            *args,
            picture_abspath=pic,
            similarity=similarity,
            return_first=return_first,
            lang=lang,
            network_retry=network_retry,
            pause=pause,
            timeout=timeout,
            max_match_number=max_match_number,
            bbox=bbox,
        )
        print(res.result)
        if res.result is False:
            pass
        elif isinstance(res.result, tuple):
            raise AssertionError(
                (
                    f"通过ocr识别到不应存在的文案 {args} -> {res.result}",
                    f"{pic if pic else pylinuxauto_config.SCREEN_CACHE}",
                )
            )
        elif isinstance(res.result, dict):
            if all(value is False for value in res.result.values()):
                pass
            else:
                if bbox is not None:
                    new_res = {}
                    for key, value in res.result.items():
                        if isinstance(value, tuple):
                            new_res[key] = (
                                value[0] + bbox.get("start_x"),
                                value[1] + bbox.get("start_y"),
                            )
                        else:
                            new_res[key] = value
                else:
                    new_res = res.result
                raise AssertionError(
                    (
                        f"通过OCR识别到不应存在的文案：{args} -> {dict(new_res)}",
                        f"{pic if pic else pylinuxauto_config.SCREEN_CACHE}",
                    )
                )



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import inspect
from socketserver import ThreadingMixIn
from xmlrpc.server import SimpleXMLRPCServer


class ThreadXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer): ...


def server(obj, port):
    server = ThreadXMLRPCServer(("0.0.0.0", port), allow_none=True)
    for func_name, _ in inspect.getmembers(obj, predicate=inspect.isfunction):
        if not func_name.startswith("_"):
            server.register_function(getattr(obj, func_name), func_name)
    server.serve_forever()



from youqu3 import logger
from youqu3.exceptions import YouQuPluginDependencyError

try:
    import requests

    HAS_REQUESTS = True
except ImportError:
    HAS_REQUESTS = False

if HAS_REQUESTS is False:
    raise YouQuPluginDependencyError("requests")


class Session:

    def __init__(self) -> None:
        self.session = requests.Session()

    def get(self, url, params=None, **kwargs):
        logger.debug(f'GET {url}')
        if params:
            logger.debug(f'params {params}')
        if kwargs:
            logger.debug(kwargs)
        return self.session.get(url, params=params, **kwargs)

    def post(self, url, data=None, json=None, **kwargs):
        logger.debug(f'POST {url}')
        if data:
            logger.debug(f'data {data}')
        if json:
            logger.debug(f'json {json}')
        if kwargs:
            logger.debug(kwargs)
        return self.session.post(url, data=data, json=json, **kwargs)

if __name__ == '__main__':
    Session().post(url="").json()


import dataclasses
import enum
import os

from youqu3._setting._dynamic import _DynamicSetting


class _Setting(_DynamicSetting):
    """YouQu Config"""

    PASSWORD: str = os.environ.get("YOUQU_PASSWORD") or "1"

    TIMEOUT = os.environ.get("YOUQU_TIMEOUT") or 300
    LOG_LEVEL = os.environ.get("YOUQU_LOG_LEVEL") or "INFO"
    RERUNS = os.environ.get("YOUQU_RERUNS") or 1
    RECORD_FAILED_NUM = os.environ.get("YOUQU_RECORD_FAILED_NUM") or 1

    # OCR
    OCR_NETWORK_RETRY = 1
    OCR_PAUSE = 1
    OCR_TIMEOUT = 5
    OCR_MAX_MATCH_NUMBER = 100
    OCR_PORT = 8890
    OCR_SERVER_IP = "10.8.13.7/10.8.13.66/10.8.13.55/10.8.13.100"

    # IMAGE
    IMAGE_NETWORK_RETRY = 1
    IMAGE_PAUSE = 1
    IMAGE_TIMEOUT = 5
    IMAGE_MAX_MATCH_NUMBER = 100
    IMAGE_PORT = 8889
    IMAGE_SERVER_IP = "10.8.12.175/10.8.12.130/10.8.12.181/10.8.12.216"

    # REPORT SERVER
    REPORT_SERVER_IP = "10.8.12.47/10.8.12.37"
    REPORT_PORT = 5656
    REPORT_BASE_PATH = "~/report"

    # REMOTE

    # SLAVES
    SLAVES = os.environ.get("SLAVES")

    # WEBUI
    EXECUTABLE_PATH = f"{_DynamicSetting.HOME}/.config/browser"
    USER_DATE_DIR = "/usr/bin/browser"
    HEADLESS = True if os.environ.get("HEADLESS") is None else False

    PYPI_MIRROR = "https://pypi.tuna.tsinghua.edu.cn/simple"

    @dataclasses.dataclass
    class Sleepx:
        x86_64: [float, int] = 1
        aarch64: [float, int] = 1.5
        loongarch64: [float, int] = 2
        mips64: [float, int] = 2.5
        sw64: [float, int] = 2.5

    @enum.unique
    class FixedCsvTitle(enum.Enum):
        case_id = "脚本ID"
        skip_reason = "跳过原因"
        fixed = "确认修复"
        removed = "废弃用例"


setting = _Setting()



from youqu3 import setting
from youqu3.exceptions import YouQuPluginDependencyError

try:

    from pylinuxauto.config import config as pylinuxauto_config

    pylinuxauto_config.OCR_SERVER_IP = setting.OCR_SERVER_IP
    pylinuxauto_config.IMAGE_SERVER_IP = setting.IMAGE_SERVER_IP

    import pylinuxauto as pylinuxauto
    from youqu3.gui.rpc_gui import RpcGui

except ImportError:
    raise YouQuPluginDependencyError("pylinuxauto")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from funnylog2 import log as log
from funnylog2 import logger as logger

from youqu3._setting import setting as setting
from youqu3.sleepx import sleep
from youqu3.__version__ import __version__ as version

__all__ = ['setting', 'log', 'logger', 'sleep', 'version']



import enum
import re

import pytest
from youqu3 import exceptions
from youqu3 import logger
from youqu3 import setting

FLAG_FEEL = "=" * 10


@enum.unique
class ConfStr(enum.Enum):
    SKIP = "skip"
    SKIPIF = "skipif"
    FIXED = "fixed"
    PASSED = "passed"
    FAILED = "failed"
    SKIPPED = "skipped"
    REMOVED = "removed"
    RERUN = "rerun"


def pytest_addoption(parser):
    parser.addoption("--slaves", action="store", default="", help="")


def pytest_sessionstart(session):
    logger("DEBUG")


def pytest_collection_finish(session):
    session.item_count = len(session.items)
    _items = session.items[:]
    is_skiped_case = False
    for item in _items[::-1]:
        for mark in item.own_markers:
            if mark.name == ConfStr.SKIP.value:
                is_skiped_case = True
                try:
                    _items.remove(item)
                except ValueError:
                    ...
            elif mark.name == ConfStr.SKIPIF.value and mark.args == (True,):
                is_skiped_case = True
                try:
                    _items.remove(item)
                except ValueError:
                    ...
    print(
        f"用例收集数量:\t{session.item_count} "
        f"{f'(剔除跳过: {len(_items)})' if is_skiped_case else ''}"
    )
    print(
        f"用例文件数量:\t{len(set([item.fspath for item in session.items]))} "
        f"{f'(剔除跳过: {len(set([item.fspath for item in _items]))})' if is_skiped_case else ''}"
    )


def pytest_runtest_setup(item):
    print()
    LN = "\n"
    current_item_count = f"[{item.session.items.index(item) + 1}/{item.session.item_count}] "
    try:
        current_item_percent = "{:.0f}%".format(
            int(item.session.items.index(item) + 1) / int(item.session.item_count) * 100
        )
    except:
        current_item_percent = ""
    try:
        rerun_text = f" | <重跑第{item.execution_count - 1}次>" if item.execution_count > 1 else ""
    except AttributeError:
        rerun_text = ""
    logger.info(
        f"{LN}{FLAG_FEEL} {item.function.__name__} | "
        f"{str(item.function.__doc__).replace(LN, '').replace('    ', '')}{rerun_text} "
        f"{FLAG_FEEL} {current_item_count} {current_item_percent}"
    )


def pytest_runtest_call(item):
    logger.info(f"{FLAG_FEEL} case body {FLAG_FEEL}")


def pytest_runtest_teardown(item):
    logger.info(f"{FLAG_FEEL} teardown {FLAG_FEEL}")


@pytest.fixture(scope='module')
def page():
    try:
        from playwright.sync_api import sync_playwright
    except ImportError:
        raise exceptions.YouQuPluginDependencyError("playwright")
    driver = sync_playwright().start()
    browser = driver.chromium.launch_persistent_context(
        user_data_dir=setting.USER_DATE_DIR,
        executable_path=setting.EXECUTABLE_PATH,
        ignore_https_errors=True,
        no_viewport=True,
        slow_mo=500,
        headless=setting.HEADLESS,
        bypass_csp=True,
        args=[
            '--disable-blink-features=AutomationControlled',
            '--start-maximized',
        ],

    )
    _page = browser.pages[0]
    yield _page
    browser.close()
    driver.stop()


@pytest.fixture(scope="session")
def slaves(pytestconfig):
    _slaves = pytestconfig.getoption("slaves") or setting.SLAVES
    s = []
    if _slaves:
        for slave in _slaves.split("/"):
            slave_info = re.findall(r"^(.+?)@(\d+\.\d+\.\d+\.\d+):{0,1}(.*?)$", slave)
            if slave_info:
                user, ip, password = slave_info[0]
                s.append(
                    {
                        "user": user,
                        "ip": ip,
                        "password": password or setting.PASSWORD,
                    }
                )
    if not s:
        raise EnvironmentError("No slaves found, check -s/--slaves value")
    return s


@pytest.fixture(scope="session")
def gui():
    from youqu3.gui import pylinuxauto
    return pylinuxauto


@pytest.fixture(scope="session")
def sleep():
    from youqu3.sleepx import sleep as slp
    return slp


@pytest.fixture(scope="session")
def cmd():
    from youqu3.cmd import Cmd
    return Cmd

@pytest.fixture(scope="session")
def check():
    from youqu3.assertx import Assert
    return Assert


import os.path
import pathlib

from youqu3 import exceptions
from youqu3 import logger
from youqu3 import setting
from youqu3.cmd import Cmd


def cli_cmd_creator(cmd_name, cmd_txt):
    cmd_path = os.path.expanduser(f"~/.local/bin/{cmd_name}")
    if not os.path.exists(cmd_path):
        Cmd.run(f"echo '{cmd_txt}' >> {cmd_path}")
        Cmd.run(f"chmod +x {cmd_path}")


def envx(python_version=None):
    if python_version is None:
        python_version = "3"
    rootdir = pathlib.Path(".").absolute()
    logger.info(rootdir)
    # pip
    _, return_code = Cmd.run("pip3 --version", return_code=True)
    if return_code != 0:
        Cmd.run(
            'curl -sSL https://bootstrap.pypa.io/get-pip.py -o get-pip.py && && python3 get-pip.py',
            timeout=600
        )
    # youqu3
    Cmd.run(f'pip3 install youqu3 -i {setting.PYPI_MIRROR} --break-system-packages')
    # youqu3-cargo
    cli_cmd_creator("youqu3-cargo", 'pipenv run youqu3 "$@"')
    # youqu3-shell
    cli_cmd_creator("youqu3-shell", 'pipenv shell')
    # youqu3-rm
    cli_cmd_creator("youqu3-rm", 'pipenv --rm')
    # PATH
    with open(os.path.expanduser("~/.bashrc"), 'r') as f:
        bashrc = f.read()
    path_cmd = "export PATH=$PATH:$HOME/.local/bin"
    if path_cmd not in bashrc:
        Cmd.run(f"echo '{path_cmd}' >> ~/.bashrc")
        Cmd.run("source ~/.bashrc")
    # pipenv
    Cmd.run(f'pip3 install pipenv -i {setting.PYPI_MIRROR} --break-system-packages')
    Cmd.run(f"cd {rootdir} && pipenv --python {python_version}")
    os.system(
        f"cd {rootdir} && pipenv run pip install -r requirements.txt -i {setting.PYPI_MIRROR} --break-system-packages",
    )


if __name__ == '__main__':
    envx()



import os
import subprocess
import sys

from youqu3 import exceptions
from youqu3 import logger
from youqu3 import setting


class Cmd:

    @staticmethod
    def _run(command, _input=None, timeout=None, check=False, executable=None, **kwargs):
        with subprocess.Popen(command, **kwargs) as process:
            try:
                stdout, stderr = process.communicate(_input, timeout=timeout)
            except:
                process.kill()
                raise
            retcode = process.poll()
            if check and retcode:
                raise subprocess.CalledProcessError(
                    retcode, process.args, output=stdout, stderr=stderr
                )
        return subprocess.CompletedProcess(process.args, retcode, stdout, stderr)

    @classmethod
    def _getstatusoutput(cls, command, timeout, executable):
        kwargs = {
            "shell": True,
            "stderr": subprocess.STDOUT,
            "stdout": subprocess.PIPE,
            "timeout": timeout,
            "executable": executable,
        }
        try:
            if sys.version_info >= (3, 7):
                kwargs["text"] = True
            result = cls._run(command, **kwargs)
            data = result.stdout
            if isinstance(data, bytes):
                data = data.decode("utf-8")
            exitcode = result.returncode
        except subprocess.CalledProcessError as ex:
            data = ex.output
            exitcode = ex.returncode
        except subprocess.TimeoutExpired as ex:
            data = ex.__str__()
            exitcode = -1
        if data[-1:] == "\n":
            data = data[:-1]
        return exitcode, data

    @classmethod
    def run(
            cls,
            command: str,
            interrupt: bool = False,
            timeout: [None, int] = 25,
            print_log: bool = True,
            command_log: bool = True,
            return_code: bool = False,
            executable: str = "/bin/bash",
    ):
        """
         执行shell命令
        :param command: shell 命令
        :param interrupt: 命令异常时是否中断
        :param timeout: 命令执行超时
        :param out_debug_flag: 命令返回信息输出日志
        :param command_log: 执行的命令字符串日志
        :return: 返回终端输出
        """
        exitcode, stdout = cls._getstatusoutput(command, timeout=timeout, executable=executable)
        if command_log:
            logger.debug(command)
        if exitcode != 0 and interrupt:
            raise exceptions.ShellExecutionFailed(stdout)
        if print_log and stdout:
            logger.debug(stdout)
        if return_code:
            return stdout, exitcode
        return stdout

    @staticmethod
    def expect_run(
            cmd: str,
            events: dict,
            return_code=False,
            timeout: int = 30
    ):
        """
        expect_run(
            "ssh username@machine_ip 'ls -l'",
            events={'password':'secret\n'}
        )
        如果 return_code=True，返回 (stdout, return_code)
        """
        import pexpect
        res = pexpect.run(
            cmd,
            events=events,
            withexitstatus=return_code,
            timeout=timeout
        )
        if return_code is False:
            return res.decode("utf-8")
        stdout, return_code = res
        return stdout.decode("utf-8"), return_code

    @classmethod
    def sudo_run(
            cls,
            command,
            password: str = None,
            workdir: str = None,
            interrupt: bool = False,
            timeout: int = 25,
            print_log: bool = True,
            command_log: bool = True,
            return_code: bool = False
    ):
        if password is None:
            password = setting.PASSWORD
        wd = ""
        if workdir:
            if not os.path.exists(workdir):
                raise FileNotFoundError
            wd = f"cd {workdir} && "
        res = cls.run(
            f"{wd}echo '{password}' | sudo -S {command}",
            interrupt=interrupt,
            timeout=timeout,
            print_log=print_log,
            command_log=command_log,
            return_code=return_code
        )
        if return_code is False:
            return res.lstrip("请输入密码●")
        res = list(res)
        res[0] = res[0].lstrip("请输入密码●")
        return res


class RemoteCmd:

    def __init__(self, user: str, ip: str, password: str):
        self.user = user
        self.ip = ip
        self.password = password

    def remote_run(self, cmd: str, return_code: bool = False, timeout: int = None):
        res = Cmd.expect_run(
            f'ssh -o StrictHostKeyChecking=no {self.user}@{self.ip} "{cmd}"',
            events={'password': f'{self.password}\n'},
            return_code=return_code,
            timeout=timeout
        )
        return res

    def remote_sudo_run(self, cmd: str, workdir: str = None, return_code: bool = False):
        wd = ""
        if workdir is not None:
            _, code = self.remote_run(f"ls {workdir}", return_code=True)
            if code == 0:
                wd = workdir
            else:
                raise FileNotFoundError(workdir)
        return self.remote_run(f"{wd}echo '{self.password}' | sudo -S {cmd}", return_code=return_code)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import pathlib

from youqu3.gui._rpc.client import client
from youqu3.gui._rpc.guard import guard_rpc
from youqu3.gui.method import RpcMethods

port = 4242


@guard_rpc(os.path.splitext(pathlib.Path(__file__).name)[0])
def _rpc_gui_client(
        user=None,
        ip=None,
        password=None,
        auto_restart=False,
        project_abspath=None
):
    return client(ip=ip, port=port)


class RpcGui:

    def __init__(
            self,
            user,
            ip,
            password,
            project_abspath,
            auto_restart=False,
    ):
        self.user = user
        self.ip = ip
        self.password = password
        self.project_abspath = project_abspath
        self.auto_restart = auto_restart

    @property
    def gui(self) -> RpcMethods:
        return _rpc_gui_client(
            user=self.user,
            ip=self.ip,
            password=self.password,
            auto_restart=self.auto_restart,
            project_abspath=self.project_abspath
        )


if __name__ == '__main__':
    from youqu3.gui._rpc.server import server

    server(RpcMethods, port)



from youqu3.gui import pylinuxauto


class RpcMethods:

    # attr
    def click_element_by_attr_path(self, attr_path):
        pylinuxauto.find_element_by_attr_path(attr_path).click()

    def double_click_element_by_attr_path(self, attr_path):
        pylinuxauto.find_element_by_attr_path(attr_path).double_click()

    def right_click_element_by_attr_path(self, attr_path):
        pylinuxauto.find_element_by_attr_path(attr_path).right_click()

    def element_center_by_attr_path(self, attr_path):
        return pylinuxauto.find_element_by_attr_path(attr_path).center()

    # image
    def click_element_by_image(self, image_path):
        pylinuxauto.find_element_by_image(image_path).click()

    def double_click_element_by_image(self, image_path):
        pylinuxauto.find_element_by_image(image_path).double_click()

    def right_click_element_by_image(self, image_path):
        pylinuxauto.find_element_by_image(image_path).right_click()

    def element_center_by_image(self, image_path):
        return pylinuxauto.find_element_by_image(image_path).center()

    # ocr
    def click_element_by_ocr(self, target):
        pylinuxauto.find_element_by_ocr(target).click()

    def double_click_element_by_ocr(self, target):
        pylinuxauto.find_element_by_ocr(target).double_click()

    def right_click_element_by_ocr(self, target):
        pylinuxauto.find_element_by_ocr(target).right_click()

    def element_center_by_ocr(self, target):
        return pylinuxauto.find_element_by_ocr(target).center()

    # ui
    def click_element_by_ui(self, appname, config_path, btn_name):
        pylinuxauto.find_element_by_ui(appname, config_path, btn_name).click()

    def double_click_element_by_ui(self, appname, config_path, btn_name):
        pylinuxauto.find_element_by_ui(appname, config_path, btn_name).double_click()

    def right_click_element_by_ui(self, appname, config_path, btn_name):
        pylinuxauto.find_element_by_ui(appname, config_path, btn_name).right_click()

    # mousekey
    def click(self, x=None, y=None):
        pylinuxauto.click(_x=x, _y=y)

    def double_click(self, x=None, y=None):
        pylinuxauto.double_click(_x=x, _y=y)

    def right_click(self, x=None, y=None):
        pylinuxauto.right_click(_x=x, _y=y)

    def move_to(self, x=None, y=None):
        pylinuxauto.move_to(_x=x, _y=y)

    def input(self, text: str):
        pylinuxauto.input(text)

    def hotkey(self, *key):
        pylinuxauto.hot_key(*key)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only

from youqu3 import logger


class ApplicationStartError(Exception):

    def __init__(self, result):
        err = f"应用程序未启动: {result}"
        logger.error(err)
        Exception.__init__(self, err)


class ElementFound(Exception):

    def __init__(self, name):
        err = f"找到元素: {name}"
        logger.error(err)
        Exception.__init__(self, err)


class ElementNotFound(Exception):

    def __init__(self, name):
        err = f"未找到元素: {name}"
        logger.error(err)
        Exception.__init__(self, err)


class TemplateElementNotFound(Exception):

    def __init__(self, *name):
        err = "未在屏幕上匹配到图片: "
        template = [f"{i}.png" for i in name]
        logger.error(f"{err}{str(*template)}")
        Exception.__init__(self, err, *template)


class TemplateElementFound(Exception):

    def __init__(self, *name):
        err = "在屏幕中匹配到了不应该出现的元素: "
        template = [f"{i}.png" for i in name]
        logger.error(f"{err}{str(*template)}")
        Exception.__init__(self, err, *template)


class TemplatePictureNotExist(Exception):

    def __init__(self, name):
        err = f"图片文件不存在: {name} "
        logger.error(err)
        Exception.__init__(self, err)


class ParamError(Exception):

    def __init__(self, name, msg):
        err = f"参数错误：{name}、\n{msg}"
        logger.error(err)
        Exception.__init__(self, err)


class ShellExecutionFailed(Exception):

    def __init__(self, msg):
        err = f"Shell执行失败: {msg}"
        logger.error(err)
        Exception.__init__(self, err)


class YouQuPluginDependencyError(Exception):

    def __init__(self, msg):
        err = f"YouQu3插件未安装 {msg}, 请尝试安装: pip install {msg}"
        logger.error(err)
        Exception.__init__(self, err)



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
from xmlrpc.client import ServerProxy


def client(ip, port):
    client = ServerProxy(f"http://{ip}:{port}", allow_none=True)
    return client
